=head1 NAME

Tails::IUK::UpgradeDescriptionFile::Download - download and verify an upgrade description file

=cut

package Tails::IUK::UpgradeDescriptionFile::Download;

use Moose;
use Moose::Util::TypeConstraints qw{class_type};
use MooseX::Method::Signatures;
use MooseX::Types::Moose qw( :all );
use MooseX::Has::Sugar::Saccharin;

with 'MooseX::Getopt::Dashes';

use 5.10.1;
use namespace::autoclean;
use warnings;

extends 'Tails::Download::HTTPS';

use autodie qw(:all);
use Carp;
use Carp::Assert;
use Carp::Assert::More;
use English qw{-no_match_vars};
use Tails::RunningSystem;
use Tails::IUK::Utils;
use YAML::Any;


=head1 TYPES

=cut

class_type 'Tails::RunningSystem';


=head1 ATTRIBUTES

=cut

has "$_" => lazy_build ro Str
    for (qw{override_baseurl override_build_target override_os_release_file
            trusted_gnupg_homedir});
has 'running_system' =>
    lazy_build ro 'Tails::RunningSystem',
    handles => [
        qw{upgrade_description_file_url upgrade_description_sig_url},
        qw{product_name product_version build_target channel}
    ];


=head1 CONSTRUCTORS AND BUILDERS

=cut

method _build_trusted_gnupg_homedir     {
    my $trusted_gnupg_homedir = '/usr/share/tails-iuk/trusted_gnupg_homedir';
    assert(-d $trusted_gnupg_homedir);
    return $trusted_gnupg_homedir;
}

method _build_running_system {
    my @args;
    for (qw{baseurl build_target os_release_file}) {
        my $attribute = "override_$_";
        my $predicate = "has_$attribute";
        if ($self->$predicate) {
            push @args, ($_ => $self->$attribute)
        }
    }
    Tails::RunningSystem->new(@args);
}


=head1 METHODS

=cut

method matches_running_system ($description_str) {
    assert(defined $description_str);
    my $description = YAML::Any::Load($description_str);
    assert_hashref($description);
    foreach (qw{product_name product_version build_target channel}) {
        my $accessor = my $field = $_;
        $field =~ s{_}{-}gxms;
        exists  $description->{$field}             or return;
        defined $description->{$field}             or return;
        $description->{$field} eq $self->$accessor or return;
    }
    return 1;
}

method run () {
    my $description = $self->get_url($self->upgrade_description_file_url);
    my $signature   = $self->get_url($self->upgrade_description_sig_url );

    verify_signature($description, $signature, $self->trusted_gnupg_homedir)
        or croak("Invalid signature");
    $self->matches_running_system($description)
        or croak("Does not match running system");
    print $description;
    return;
}

no Moose;
1;
